@@ -34,6 +34,8 @@ else:
 import urllib3
 import time
 import datetime
+import hashlib
+from version import VERSION
 
 POOL_MANAGER = urllib3.PoolManager()
 
@@ -68,7 +70,7 @@ class Client:
             'Content-Type': 'application/json; charset=utf-8',
             'X-Algolia-API-Key': self.apiKey,
             'X-Algolia-Application-Id': self.applicationID,
-            'User-Agent': 'Algolia Search for python'
+            'User-Agent': ('Algolia Search for python %s' % VERSION)
         }
 
     def enableRateLimitForward(self, adminAPIKey, endUserIP, rateLimitAPIKey):
@@ -115,7 +117,7 @@ class Client:
         @param indexName the name of index to delete
         Return an object of the form {"deletedAt": "2013-01-18T15:33:13.556Z"}
         """
-        return AlgoliaUtils_request(self.headers, self.hosts, "DELETE", "/1/indexes/%s" % quote(indexName, safe=''))
+        return AlgoliaUtils_request(self.headers, self.hosts, "DELETE", "/1/indexes/%s" % quote(indexName.encode('utf8'), safe=''))
 
     def moveIndex(self, srcIndexName, dstIndexName):
         """
@@ -172,7 +174,7 @@ class Client:
         """
         return AlgoliaUtils_request(self.headers, self.hosts, "DELETE", "/1/keys/%s" % key)
 
-    def addUserKey(self, acls, validity = 0, maxQueriesPerIPPerHour = 0, maxHitsPerQuery = 0):
+    def addUserKey(self, acls, validity = 0, maxQueriesPerIPPerHour = 0, maxHitsPerQuery = 0, indexes = None):
         """
         Create a new user key
 
@@ -187,8 +189,25 @@ class Client:
         @param validity the number of seconds after which the key will be automatically removed (0 means no time limit for this key)
         @param maxQueriesPerIPPerHour Specify the maximum number of API calls allowed from an IP address per hour.  Defaults to 0 (no rate limit).
         @param maxHitsPerQuery Specify the maximum number of hits this API key can retrieve in one call. Defaults to 0 (unlimited)
+        @param indexes the optional list of targeted indexes
         """
-        return AlgoliaUtils_request(self.headers, self.hosts, "POST", "/1/keys", {"acl": acls, "validity": validity, "maxQueriesPerIPPerHour": maxQueriesPerIPPerHour, "maxHitsPerQuery": maxHitsPerQuery} )
+        params = {"acl": acls, "validity": validity, "maxQueriesPerIPPerHour": maxQueriesPerIPPerHour, "maxHitsPerQuery": maxHitsPerQuery}
+        if not indexes is None:
+            params['indexes'] = indexes
+        return AlgoliaUtils_request(self.headers, self.hosts, "POST", "/1/keys", params)
+
+    def generateSecuredApiKey(self, private_api_key, tag_filters, user_token = None):
+        """
+        Generate a secured and public API Key from a list of tagFilters and an
+        optional user token identifying the current user
+ 
+        @param private_api_key your private API Key
+        @param tag_filters the list of tags applied to the query (used as security)
+        @param user_token an optional token identifying the current user
+        """
+        if type(tag_filters) is list:
+            tag_filters = ','.join(map(lambda t: ''.join(['(', ','.join(t), ')']) if type(t) is list else str(t), tag_filters))
+        return hashlib.sha256(''.join([private_api_key, str(tag_filters), str(user_token or '')])).hexdigest()
 
 class Index:
     """
@@ -282,6 +301,18 @@ class Index:
         request = {"requests": requests}
         return self.batch(request)
 
+    def deleteObjects(self, objects):
+        """
+        delete several objects
+
+        @param objects contains an array of objectID to delete
+        """
+        requests = []
+        for obj in objects:
+            requests.append({"action": "deleteObject", "body": { "objectID" : obj}})
+        request = {"requests": requests}
+        return self.batch(request)
+
     def deleteObject(self, objectID):
         """
         Delete an object from the index
